@@ -19,8 +19,6 @@
 package org.apache.catalina.ant;
 
 
-import java.io.UnsupportedEncodingException;
-import java.net.URLEncoder;
 
 import org.apache.tools.ant.BuildException;
 
@@ -30,51 +28,21 @@ import org.apache.tools.ant.BuildException;
  * Tomcat manager application.
  *
  * @author Craig R. McClanahan
- * @version $Revision: 467222 $ $Date: 2006-10-24 05:17:11 +0200 (Di, 24. Okt 2006) $
+ * @version $Id: ReloadTask.java 1131267 2011-06-03 22:22:35Z markt $
  * @since 4.1
  */
-public class ReloadTask extends AbstractCatalinaTask {
-
-
-    // ------------------------------------------------------------- Properties
-
-
-    /**
-     * The context path of the web application we are managing.
-     */
-    protected String path = null;
-
-    public String getPath() {
-        return (this.path);
-    }
-
-    public void setPath(String path) {
-        this.path = path;
-    }
-
-
-    // --------------------------------------------------------- Public Methods
-
+public class ReloadTask extends AbstractCatalinaCommandTask {
 
     /**
      * Execute the requested operation.
      *
      * @exception BuildException if an error occurs
      */
+    @Override
     public void execute() throws BuildException {
 
         super.execute();
-        if (path == null) {
-            throw new BuildException
-                ("Must specify 'path' attribute");
-        }
-        try {
-            execute("/reload?path=" + URLEncoder.encode(this.path, getCharset()));
-        } catch (UnsupportedEncodingException e) {
-            throw new BuildException
-                ("Invalid 'charset' attribute: " + getCharset());
-        }
-
+        execute(createQueryString("/reload").toString());
 
     }
 
