@@ -5,90 +5,98 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
-
 package org.apache.catalina.connector;
 
 import java.io.BufferedReader;
 import java.io.IOException;
 import java.security.AccessController;
 import java.security.PrivilegedAction;
+import java.util.Collection;
 import java.util.Enumeration;
 import java.util.Locale;
 import java.util.Map;
 
+import javax.servlet.AsyncContext;
+import javax.servlet.DispatcherType;
 import javax.servlet.RequestDispatcher;
+import javax.servlet.ServletContext;
+import javax.servlet.ServletException;
 import javax.servlet.ServletInputStream;
+import javax.servlet.ServletRequest;
+import javax.servlet.ServletResponse;
 import javax.servlet.http.Cookie;
 import javax.servlet.http.HttpServletRequest;
+import javax.servlet.http.HttpServletResponse;
 import javax.servlet.http.HttpSession;
+import javax.servlet.http.Part;
 
 import org.apache.catalina.Globals;
-import org.apache.catalina.util.StringManager;
-
-
 import org.apache.catalina.security.SecurityUtil;
+import org.apache.tomcat.util.res.StringManager;
 
 /**
- * Facade class that wraps a Coyote request object.  
+ * Facade class that wraps a Coyote request object.
  * All methods are delegated to the wrapped request.
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
  * @author Jean-Francois Arcand
- * @version $Id: RequestFacade.java 939336 2010-04-29 15:00:41Z kkolinko $
+ * @version $Id: RequestFacade.java 1200178 2011-11-10 06:14:46Z kkolinko $
  */
 
 @SuppressWarnings("deprecation")
 public class RequestFacade implements HttpServletRequest {
-        
-        
-    // ----------------------------------------------------------- DoPrivileged
-    
-    private final class GetAttributePrivilegedAction
-            implements PrivilegedAction {
-        
-        public Object run() {
-            return request.getAttributeNames();
-        }            
-    }
-     
-    
-    private final class GetParameterMapPrivilegedAction
-            implements PrivilegedAction {
-        
-        public Object run() {
-            return request.getParameterMap();
-        }        
-    }    
-    
-    
-    private final class GetRequestDispatcherPrivilegedAction
-            implements PrivilegedAction {
 
-        private String path;
+
+    // ----------------------------------------------------------- DoPrivileged
+
+    private final class GetAttributePrivilegedAction
+            implements PrivilegedAction<Enumeration<String>> {
+
+        @Override
+        public Enumeration<String> run() {
+            return request.getAttributeNames();
+        }
+    }
+
+
+    private final class GetParameterMapPrivilegedAction
+            implements PrivilegedAction<Map<String,String[]>> {
+
+        @Override
+        public Map<String,String[]> run() {
+            return request.getParameterMap();
+        }
+    }
+
+
+    private final class GetRequestDispatcherPrivilegedAction
+            implements PrivilegedAction<RequestDispatcher> {
+
+        private final String path;
 
         public GetRequestDispatcherPrivilegedAction(String path){
             this.path = path;
         }
-        
-        public Object run() {   
+
+        @Override
+        public RequestDispatcher run() {
             return request.getRequestDispatcher(path);
-        }           
-    }    
-    
-    
+        }
+    }
+
+
     private final class GetParameterPrivilegedAction
-            implements PrivilegedAction {
+            implements PrivilegedAction<String> {
 
         public String name;
 
@@ -96,23 +104,25 @@ public class RequestFacade implements HttpServletRequest {
             this.name = name;
         }
 
-        public Object run() {       
+        @Override
+        public String run() {
             return request.getParameter(name);
-        }           
-    }    
-    
-     
+        }
+    }
+
+
     private final class GetParameterNamesPrivilegedAction
-            implements PrivilegedAction {
-        
-        public Object run() {          
+            implements PrivilegedAction<Enumeration<String>> {
+
+        @Override
+        public Enumeration<String> run() {
             return request.getParameterNames();
-        }           
-    } 
-    
-    
+        }
+    }
+
+
     private final class GetParameterValuePrivilegedAction
-            implements PrivilegedAction {
+            implements PrivilegedAction<String[]> {
 
         public String name;
 
@@ -120,83 +130,91 @@ public class RequestFacade implements HttpServletRequest {
             this.name = name;
         }
 
-        public Object run() {       
+        @Override
+        public String[] run() {
             return request.getParameterValues(name);
-        }           
-    }    
-  
-    
-    private final class GetCookiesPrivilegedAction
-            implements PrivilegedAction {
-        
-        public Object run() {       
-            return request.getCookies();
-        }           
-    }      
-    
-    
-    private final class GetCharacterEncodingPrivilegedAction
-            implements PrivilegedAction {
-        
-        public Object run() {       
-            return request.getCharacterEncoding();
-        }           
-    }   
-        
-    
-    private final class GetHeadersPrivilegedAction
-            implements PrivilegedAction {
+        }
+    }
 
-        private String name;
+
+    private final class GetCookiesPrivilegedAction
+            implements PrivilegedAction<Cookie[]> {
+
+        @Override
+        public Cookie[] run() {
+            return request.getCookies();
+        }
+    }
+
+
+    private final class GetCharacterEncodingPrivilegedAction
+            implements PrivilegedAction<String> {
+
+        @Override
+        public String run() {
+            return request.getCharacterEncoding();
+        }
+    }
+
+
+    private final class GetHeadersPrivilegedAction
+            implements PrivilegedAction<Enumeration<String>> {
+
+        private final String name;
 
         public GetHeadersPrivilegedAction(String name){
             this.name = name;
         }
-        
-        public Object run() {       
+
+        @Override
+        public Enumeration<String> run() {
             return request.getHeaders(name);
-        }           
-    }    
-        
-    
+        }
+    }
+
+
     private final class GetHeaderNamesPrivilegedAction
-            implements PrivilegedAction {
+            implements PrivilegedAction<Enumeration<String>> {
 
-        public Object run() {       
+        @Override
+        public Enumeration<String> run() {
             return request.getHeaderNames();
-        }           
-    }  
-            
-    
+        }
+    }
+
+
     private final class GetLocalePrivilegedAction
-            implements PrivilegedAction {
+            implements PrivilegedAction<Locale> {
 
-        public Object run() {       
+        @Override
+        public Locale run() {
             return request.getLocale();
-        }           
-    }    
-            
-    
+        }
+    }
+
+
     private final class GetLocalesPrivilegedAction
-            implements PrivilegedAction {
+            implements PrivilegedAction<Enumeration<Locale>> {
 
-        public Object run() {       
+        @Override
+        public Enumeration<Locale> run() {
             return request.getLocales();
-        }           
-    }    
-    
+        }
+    }
+
     private final class GetSessionPrivilegedAction
-            implements PrivilegedAction {
+            implements PrivilegedAction<HttpSession> {
+
+        private final boolean create;
 
-        private boolean create;
-        
         public GetSessionPrivilegedAction(boolean create){
             this.create = create;
         }
-                
-        public Object run() {  
+
+        @Override
+        public HttpSession run() {
             return request.getSession(create);
-        }           
+        }
     }
 
     // ----------------------------------------------------------- Constructors
@@ -226,7 +244,7 @@ public class RequestFacade implements HttpServletRequest {
     /**
      * The string manager for this package.
      */
-    protected static StringManager sm =
+    protected static final StringManager sm =
         StringManager.getManager(Constants.Package);
 
 
@@ -240,10 +258,11 @@ public class RequestFacade implements HttpServletRequest {
         request = null;
     }
 
-    
+
     /**
      * Prevent cloning the facade.
      */
+    @Override
     protected Object clone()
         throws CloneNotSupportedException {
         throw new CloneNotSupportedException();
@@ -253,6 +272,7 @@ public class RequestFacade implements HttpServletRequest {
     // ------------------------------------------------- ServletRequest Methods
 
 
+    @Override
     public Object getAttribute(String name) {
 
         if (request == null) {
@@ -264,7 +284,8 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
-    public Enumeration getAttributeNames() {
+    @Override
+    public Enumeration<String> getAttributeNames() {
 
         if (request == null) {
             throw new IllegalStateException(
@@ -272,14 +293,15 @@ public class RequestFacade implements HttpServletRequest {
         }
 
         if (Globals.IS_SECURITY_ENABLED){
-            return (Enumeration)AccessController.doPrivileged(
-                new GetAttributePrivilegedAction());        
+            return AccessController.doPrivileged(
+                new GetAttributePrivilegedAction());
         } else {
             return request.getAttributeNames();
         }
     }
 
 
+    @Override
     public String getCharacterEncoding() {
 
         if (request == null) {
@@ -288,14 +310,15 @@ public class RequestFacade implements HttpServletRequest {
         }
 
         if (Globals.IS_SECURITY_ENABLED){
-            return (String)AccessController.doPrivileged(
+            return AccessController.doPrivileged(
                 new GetCharacterEncodingPrivilegedAction());
         } else {
             return request.getCharacterEncoding();
-        }         
+        }
     }
 
 
+    @Override
     public void setCharacterEncoding(String env)
             throws java.io.UnsupportedEncodingException {
 
@@ -308,6 +331,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public int getContentLength() {
 
         if (request == null) {
@@ -319,6 +343,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public String getContentType() {
 
         if (request == null) {
@@ -330,6 +355,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public ServletInputStream getInputStream() throws IOException {
 
         if (request == null) {
@@ -341,6 +367,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public String getParameter(String name) {
 
         if (request == null) {
@@ -349,7 +376,7 @@ public class RequestFacade implements HttpServletRequest {
         }
 
         if (Globals.IS_SECURITY_ENABLED){
-            return (String)AccessController.doPrivileged(
+            return AccessController.doPrivileged(
                 new GetParameterPrivilegedAction(name));
         } else {
             return request.getParameter(name);
@@ -357,7 +384,8 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
-    public Enumeration getParameterNames() {
+    @Override
+    public Enumeration<String> getParameterNames() {
 
         if (request == null) {
             throw new IllegalStateException(
@@ -365,7 +393,7 @@ public class RequestFacade implements HttpServletRequest {
         }
 
         if (Globals.IS_SECURITY_ENABLED){
-            return (Enumeration)AccessController.doPrivileged(
+            return AccessController.doPrivileged(
                 new GetParameterNamesPrivilegedAction());
         } else {
             return request.getParameterNames();
@@ -373,6 +401,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public String[] getParameterValues(String name) {
 
         if (request == null) {
@@ -384,13 +413,13 @@ public class RequestFacade implements HttpServletRequest {
 
         /*
          * Clone the returned array only if there is a security manager
-         * in place, so that performance won't suffer in the nonsecure case
+         * in place, so that performance won't suffer in the non-secure case
          */
         if (SecurityUtil.isPackageProtectionEnabled()){
-            ret = (String[]) AccessController.doPrivileged(
+            ret = AccessController.doPrivileged(
                 new GetParameterValuePrivilegedAction(name));
             if (ret != null) {
-                ret = (String[]) ret.clone();
+                ret = ret.clone();
             }
         } else {
             ret = request.getParameterValues(name);
@@ -400,7 +429,8 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
-    public Map getParameterMap() {
+    @Override
+    public Map<String,String[]> getParameterMap() {
 
         if (request == null) {
             throw new IllegalStateException(
@@ -408,14 +438,15 @@ public class RequestFacade implements HttpServletRequest {
         }
 
         if (Globals.IS_SECURITY_ENABLED){
-            return (Map)AccessController.doPrivileged(
-                new GetParameterMapPrivilegedAction());        
+            return AccessController.doPrivileged(
+                new GetParameterMapPrivilegedAction());
         } else {
             return request.getParameterMap();
         }
     }
 
 
+    @Override
     public String getProtocol() {
 
         if (request == null) {
@@ -427,6 +458,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public String getScheme() {
 
         if (request == null) {
@@ -438,6 +470,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public String getServerName() {
 
         if (request == null) {
@@ -449,6 +482,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public int getServerPort() {
 
         if (request == null) {
@@ -460,6 +494,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public BufferedReader getReader() throws IOException {
 
         if (request == null) {
@@ -471,6 +506,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public String getRemoteAddr() {
 
         if (request == null) {
@@ -482,6 +518,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public String getRemoteHost() {
 
         if (request == null) {
@@ -493,6 +530,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public void setAttribute(String name, Object o) {
 
         if (request == null) {
@@ -504,6 +542,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public void removeAttribute(String name) {
 
         if (request == null) {
@@ -515,6 +554,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public Locale getLocale() {
 
         if (request == null) {
@@ -523,15 +563,16 @@ public class RequestFacade implements HttpServletRequest {
         }
 
         if (Globals.IS_SECURITY_ENABLED){
-            return (Locale)AccessController.doPrivileged(
+            return AccessController.doPrivileged(
                 new GetLocalePrivilegedAction());
         } else {
             return request.getLocale();
-        }        
+        }
     }
 
 
-    public Enumeration getLocales() {
+    @Override
+    public Enumeration<Locale> getLocales() {
 
         if (request == null) {
             throw new IllegalStateException(
@@ -539,14 +580,15 @@ public class RequestFacade implements HttpServletRequest {
         }
 
         if (Globals.IS_SECURITY_ENABLED){
-            return (Enumeration)AccessController.doPrivileged(
+            return AccessController.doPrivileged(
                 new GetLocalesPrivilegedAction());
         } else {
             return request.getLocales();
-        }        
+        }
     }
 
 
+    @Override
     public boolean isSecure() {
 
         if (request == null) {
@@ -558,6 +600,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public RequestDispatcher getRequestDispatcher(String path) {
 
         if (request == null) {
@@ -566,13 +609,14 @@ public class RequestFacade implements HttpServletRequest {
         }
 
         if (Globals.IS_SECURITY_ENABLED){
-            return (RequestDispatcher)AccessController.doPrivileged(
+            return AccessController.doPrivileged(
                 new GetRequestDispatcherPrivilegedAction(path));
         } else {
             return request.getRequestDispatcher(path);
         }
     }
 
+    @Override
     public String getRealPath(String path) {
 
         if (request == null) {
@@ -584,6 +628,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public String getAuthType() {
 
         if (request == null) {
@@ -595,6 +640,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public Cookie[] getCookies() {
 
         if (request == null) {
@@ -606,13 +652,13 @@ public class RequestFacade implements HttpServletRequest {
 
         /*
          * Clone the returned array only if there is a security manager
-         * in place, so that performance won't suffer in the nonsecure case
+         * in place, so that performance won't suffer in the non-secure case
          */
         if (SecurityUtil.isPackageProtectionEnabled()){
-            ret = (Cookie[])AccessController.doPrivileged(
+            ret = AccessController.doPrivileged(
                 new GetCookiesPrivilegedAction());
             if (ret != null) {
-                ret = (Cookie[]) ret.clone();
+                ret = ret.clone();
             }
         } else {
             ret = request.getCookies();
@@ -622,6 +668,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public long getDateHeader(String name) {
 
         if (request == null) {
@@ -633,6 +680,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public String getHeader(String name) {
 
         if (request == null) {
@@ -644,7 +692,8 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
-    public Enumeration getHeaders(String name) {
+    @Override
+    public Enumeration<String> getHeaders(String name) {
 
         if (request == null) {
             throw new IllegalStateException(
@@ -652,15 +701,16 @@ public class RequestFacade implements HttpServletRequest {
         }
 
         if (Globals.IS_SECURITY_ENABLED){
-            return (Enumeration)AccessController.doPrivileged(
+            return AccessController.doPrivileged(
                 new GetHeadersPrivilegedAction(name));
         } else {
             return request.getHeaders(name);
-        }         
+        }
     }
 
 
-    public Enumeration getHeaderNames() {
+    @Override
+    public Enumeration<String> getHeaderNames() {
 
         if (request == null) {
             throw new IllegalStateException(
@@ -668,14 +718,15 @@ public class RequestFacade implements HttpServletRequest {
         }
 
         if (Globals.IS_SECURITY_ENABLED){
-            return (Enumeration)AccessController.doPrivileged(
+            return AccessController.doPrivileged(
                 new GetHeaderNamesPrivilegedAction());
         } else {
             return request.getHeaderNames();
-        }             
+        }
     }
 
 
+    @Override
     public int getIntHeader(String name) {
 
         if (request == null) {
@@ -687,6 +738,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public String getMethod() {
 
         if (request == null) {
@@ -698,6 +750,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public String getPathInfo() {
 
         if (request == null) {
@@ -709,6 +762,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public String getPathTranslated() {
 
         if (request == null) {
@@ -720,6 +774,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public String getContextPath() {
 
         if (request == null) {
@@ -731,6 +786,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public String getQueryString() {
 
         if (request == null) {
@@ -742,6 +798,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public String getRemoteUser() {
 
         if (request == null) {
@@ -753,6 +810,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public boolean isUserInRole(String role) {
 
         if (request == null) {
@@ -764,6 +822,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public java.security.Principal getUserPrincipal() {
 
         if (request == null) {
@@ -775,6 +834,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public String getRequestedSessionId() {
 
         if (request == null) {
@@ -786,6 +846,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public String getRequestURI() {
 
         if (request == null) {
@@ -797,6 +858,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public StringBuffer getRequestURL() {
 
         if (request == null) {
@@ -808,6 +870,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public String getServletPath() {
 
         if (request == null) {
@@ -819,6 +882,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public HttpSession getSession(boolean create) {
 
         if (request == null) {
@@ -827,13 +891,14 @@ public class RequestFacade implements HttpServletRequest {
         }
 
         if (SecurityUtil.isPackageProtectionEnabled()){
-            return (HttpSession)AccessController.
+            return AccessController.
                 doPrivileged(new GetSessionPrivilegedAction(create));
         } else {
             return request.getSession(create);
         }
     }
 
+    @Override
     public HttpSession getSession() {
 
         if (request == null) {
@@ -845,6 +910,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public boolean isRequestedSessionIdValid() {
 
         if (request == null) {
@@ -856,6 +922,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public boolean isRequestedSessionIdFromCookie() {
 
         if (request == null) {
@@ -867,6 +934,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public boolean isRequestedSessionIdFromURL() {
 
         if (request == null) {
@@ -878,6 +946,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public boolean isRequestedSessionIdFromUrl() {
 
         if (request == null) {
@@ -889,6 +958,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public String getLocalAddr() {
 
         if (request == null) {
@@ -900,6 +970,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public String getLocalName() {
 
         if (request == null) {
@@ -911,6 +982,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public int getLocalPort() {
 
         if (request == null) {
@@ -922,6 +994,7 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
     public int getRemotePort() {
 
         if (request == null) {
@@ -932,7 +1005,84 @@ public class RequestFacade implements HttpServletRequest {
         return request.getRemotePort();
     }
 
+
+    @Override
+    public ServletContext getServletContext() {
+        if (request == null) {
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
+        }
+
+        return request.getServletContext();
+    }
+
+
+    @Override
+    public AsyncContext startAsync() throws IllegalStateException {
+        return request.startAsync();
+    }
+
+
+    @Override
+    public AsyncContext startAsync(ServletRequest request, ServletResponse response)
+    throws IllegalStateException {
+        return this.request.startAsync(request, response);
+    }
+
+
+    @Override
+    public boolean isAsyncStarted() {
+        return request.isAsyncStarted();
+    }
+
+
+    @Override
+    public boolean isAsyncSupported() {
+        return request.isAsyncSupported();
+    }
+
+
+    @Override
+    public AsyncContext getAsyncContext() {
+        return request.getAsyncContext();
+    }
+
+    @Override
+    public DispatcherType getDispatcherType() {
+        return request.getDispatcherType();
+    }
+
+    @Override
+    public boolean authenticate(HttpServletResponse response)
+    throws IOException, ServletException {
+        return request.authenticate(response);
+    }
+
+    @Override
+    public void login(String username, String password)
+    throws ServletException {
+        request.login(username, password);
+    }
+
+    @Override
+    public void logout() throws ServletException {
+        request.logout();
+    }
+
+    @Override
+    public Collection<Part> getParts() throws IllegalStateException,
+            IOException, ServletException {
+        return request.getParts();
+    }
+
+    @Override
+    public Part getPart(String name) throws IllegalStateException, IOException,
+            ServletException {
+        return request.getPart(name);
+    }
+
     public boolean getAllowTrace() {
         return request.getConnector().getAllowTrace();
     }
+
 }
