@@ -14,20 +14,17 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
-
 package org.apache.catalina.core;
 
 
 import java.util.ArrayList;
 import java.util.List;
+import java.util.Locale;
 import java.util.Map;
 import java.util.WeakHashMap;
+import java.util.concurrent.ExecutorService;
 import java.util.regex.Pattern;
 
-import javax.management.MBeanServer;
-import javax.management.ObjectName;
-
 import org.apache.catalina.Container;
 import org.apache.catalina.Context;
 import org.apache.catalina.Host;
@@ -37,9 +34,9 @@ import org.apache.catalina.LifecycleException;
 import org.apache.catalina.LifecycleListener;
 import org.apache.catalina.Valve;
 import org.apache.catalina.loader.WebappClassLoader;
-import org.apache.catalina.startup.HostConfig;
+import org.apache.catalina.mbeans.MBeanUtils;
 import org.apache.catalina.valves.ValveBase;
-import org.apache.tomcat.util.modeler.Registry;
+import org.apache.tomcat.util.ExceptionUtils;
 
 
 /**
@@ -49,16 +46,12 @@ import org.apache.tomcat.util.modeler.Registry;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Id: StandardHost.java 1060467 2011-01-18 17:01:07Z markt $
+ * @version $Id: StandardHost.java 1190190 2011-10-28 07:59:16Z markt $
  */
 
-public class StandardHost
-    extends ContainerBase
-    implements Host  
- {
-    /* Why do we implement deployer and delegate to deployer ??? */
+public class StandardHost extends ContainerBase implements Host {
 
-    private static org.apache.juli.logging.Log log=
+    private static final org.apache.juli.logging.Log log=
         org.apache.juli.logging.LogFactory.getLog( StandardHost.class );
     
     // ----------------------------------------------------------- Constructors
@@ -91,6 +84,10 @@ public class StandardHost
      */
     private String appBase = "webapps";
 
+    /**
+     * The XML root for this Host.
+     */
+    private String xmlBase = null;
 
     /**
      * The auto deploy flag for this Host.
@@ -126,6 +123,13 @@ public class StandardHost
     private boolean deployXML = true;
 
 
+    /**
+     * Should XML files be copied to $CATALINA_BASE/conf/<engine>/<host> by
+     * default when a web application is deployed?
+     */
+    private boolean copyXML = false;
+
+
     /**
      * The Java class name of the default error reporter implementation class 
      * for deployed web applications.
@@ -133,11 +137,6 @@ public class StandardHost
     private String errorReportValveClass =
         "org.apache.catalina.valves.ErrorReportValve";
 
-    /**
-     * The object name for the errorReportValve.
-     */
-    private ObjectName errorReportValveObjectName = null;
-
     /**
      * The descriptive information string for this implementation.
      */
@@ -145,12 +144,6 @@ public class StandardHost
         "org.apache.catalina.core.StandardHost/1.0";
 
 
-    /**
-     * The live deploy flag for this Host.
-     */
-    private boolean liveDeploy = true;
-
-
     /**
      * Unpack WARs property.
      */
@@ -164,23 +157,19 @@ public class StandardHost
 
 
     /**
-     * Attribute value used to turn on/off XML validation
+     * Should we create directories upon startup for appBase and xmlBase
      */
-     private boolean xmlValidation = false;
-
-
-    /**
-     * Attribute value used to turn on/off XML namespace awarenes.
-     */
-     private boolean xmlNamespaceAware = false;
+     private boolean createDirs = true;
 
+     
      /**
       * Track the class loaders for the child web applications so memory leaks
       * can be detected.
       */
      private Map<ClassLoader, String> childClassLoaders =
          new WeakHashMap<ClassLoader, String>();
-
+     
+     
      /**
       * Any file or directory in {@link #appBase} that this pattern matches will
       * be ignored by the automatic deployment process (both
@@ -191,14 +180,31 @@ public class StandardHost
 
     // ------------------------------------------------------------- Properties
 
+     @Override
+     public ExecutorService getStartStopExecutor() {
+         return startStopExecutor;
+     }
+
 
     /**
      * Return the application root for this Host.  This can be an absolute
      * pathname, a relative pathname, or a URL.
      */
+    @Override
     public String getAppBase() {
-
         return (this.appBase);
+    }
+
+
+    /**
+     * Return the XML root for this Host.  This can be an absolute
+     * pathname, a relative pathname, or a URL.
+     * If null, defaults to ${catalina.base}/conf/ directory
+     */
+    @Override
+    public String getXmlBase() {
+
+        return (this.xmlBase);
 
     }
 
@@ -209,6 +215,7 @@ public class StandardHost
      *
      * @param appBase The new application root
      */
+    @Override
     public void setAppBase(String appBase) {
 
         String oldAppBase = this.appBase;
@@ -216,12 +223,48 @@ public class StandardHost
         support.firePropertyChange("appBase", oldAppBase, this.appBase);
 
     }
+    
 
+    /**
+     * Set the Xml root for this Host.  This can be an absolute
+     * pathname, a relative pathname, or a URL.
+     * If null, defaults to ${catalina.base}/conf/ directory
+     *
+     * @param xmlBase The new XML root
+     */
+    @Override
+    public void setXmlBase(String xmlBase) {
+
+        String oldXmlBase = this.xmlBase;
+        this.xmlBase = xmlBase;
+        support.firePropertyChange("xmlBase", oldXmlBase, this.xmlBase);
+
+    }
+
+
+    /**
+     * Returns true if the Host will attempt to create directories for appBase and xmlBase
+     * unless they already exist.
+     */
+    @Override
+    public boolean getCreateDirs() {
+        return createDirs;
+    }
+
+    /**
+     * Set to true if the Host should attempt to create directories for xmlBase and appBase upon startup
+     * @param createDirs
+     */
+    @Override
+    public void setCreateDirs(boolean createDirs) {
+        this.createDirs = createDirs;
+    }
 
     /**
      * Return the value of the auto deploy flag.  If true, it indicates that 
      * this host's child webapps will be dynamically deployed.
      */
+    @Override
     public boolean getAutoDeploy() {
 
         return (this.autoDeploy);
@@ -234,6 +277,7 @@ public class StandardHost
      * 
      * @param autoDeploy The new auto deploy flag
      */
+    @Override
     public void setAutoDeploy(boolean autoDeploy) {
 
         boolean oldAutoDeploy = this.autoDeploy;
@@ -248,6 +292,7 @@ public class StandardHost
      * Return the Java class name of the context configuration class
      * for new web applications.
      */
+    @Override
     public String getConfigClass() {
 
         return (this.configClass);
@@ -261,6 +306,7 @@ public class StandardHost
      *
      * @param configClass The new context configuration class
      */
+    @Override
     public void setConfigClass(String configClass) {
 
         String oldConfigClass = this.configClass;
@@ -300,9 +346,10 @@ public class StandardHost
 
     /**
      * Return the value of the deploy on startup flag.  If true, it indicates 
-     * that this host's child webapps should be discovred and automatically 
+     * that this host's child webapps should be discovered and automatically 
      * deployed at startup time.
      */
+    @Override
     public boolean getDeployOnStartup() {
 
         return (this.deployOnStartup);
@@ -315,6 +362,7 @@ public class StandardHost
      * 
      * @param deployOnStartup The new deploy on startup flag
      */
+    @Override
     public void setDeployOnStartup(boolean deployOnStartup) {
 
         boolean oldDeployOnStartup = this.deployOnStartup;
@@ -346,27 +394,27 @@ public class StandardHost
 
 
     /**
-     * Return the value of the live deploy flag.  If true, it indicates that 
-     * a background thread should be started that looks for web application
-     * context files, WAR files, or unpacked directories being dropped in to
-     * the <code>appBase</code> directory, and deploys new ones as they are
-     * encountered.
+     * Return the copy XML config file flag for this component.
      */
-    public boolean getLiveDeploy() {
-        return (this.autoDeploy);
+    public boolean isCopyXML() {
+
+        return (this.copyXML);
+
     }
 
 
     /**
-     * Set the live deploy flag value for this host.
-     * 
-     * @param liveDeploy The new live deploy flag
+     * Set the copy XML config file flag for this component.
+     *
+     * @param copyXML The new copy XML flag
      */
-    public void setLiveDeploy(boolean liveDeploy) {
-        setAutoDeploy(liveDeploy);
+    public void setCopyXML(boolean copyXML) {
+
+        this.copyXML= copyXML;
+
     }
-
-
+    
+    
     /**
      * Return the Java class name of the error report valve class
      * for new web applications.
@@ -399,6 +447,7 @@ public class StandardHost
      * Return the canonical, fully qualified, name of the virtual host
      * this Container represents.
      */
+    @Override
     public String getName() {
 
         return (name);
@@ -414,13 +463,14 @@ public class StandardHost
      *
      * @exception IllegalArgumentException if name is null
      */
+    @Override
     public void setName(String name) {
 
         if (name == null)
             throw new IllegalArgumentException
                 (sm.getString("standardHost.nullName"));
 
-        name = name.toLowerCase();      // Internally all names are lower case
+        name = name.toLowerCase(Locale.ENGLISH);      // Internally all names are lower case
 
         String oldName = this.name;
         this.name = name;
@@ -448,45 +498,7 @@ public class StandardHost
 
     }
 
-     /**
-     * Set the validation feature of the XML parser used when
-     * parsing xml instances.
-     * @param xmlValidation true to enable xml instance validation
-     */
-    public void setXmlValidation(boolean xmlValidation){
-        
-        this.xmlValidation = xmlValidation;
 
-    }
-
-    /**
-     * Get the server.xml &lt;host&gt; attribute's xmlValidation.
-     * @return true if validation is enabled.
-     *
-     */
-    public boolean getXmlValidation(){
-        return xmlValidation;
-    }
-
-    /**
-     * Get the server.xml &lt;host&gt; attribute's xmlNamespaceAware.
-     * @return true if namespace awarenes is enabled.
-     *
-     */
-    public boolean getXmlNamespaceAware(){
-        return xmlNamespaceAware;
-    }
-
-
-    /**
-     * Set the namespace aware feature of the XML parser used when
-     * parsing xml instances.
-     * @param xmlNamespaceAware true to enable namespace awareness
-     */
-    public void setXmlNamespaceAware(boolean xmlNamespaceAware){
-        this.xmlNamespaceAware=xmlNamespaceAware;
-    }    
-    
     /**
      * Host work directory base.
      */
@@ -510,6 +522,7 @@ public class StandardHost
      * the host's {@link #appBase} that will be ignored by the automatic
      * deployment process.
      */
+    @Override
     public String getDeployIgnore() {
         if (deployIgnore == null) {
             return null;
@@ -523,6 +536,7 @@ public class StandardHost
      * directories in the host's {@link #appBase} that will be ignored by the
      * automatic deployment process.
      */
+    @Override
     public Pattern getDeployIgnorePattern() {
         return this.deployIgnore;
     }
@@ -533,6 +547,7 @@ public class StandardHost
      * the host's {@link #appBase} that will be ignored by the automatic
      * deployment process.
      */
+    @Override
     public void setDeployIgnore(String deployIgnore) {
         String oldDeployIgnore;
         if (this.deployIgnore == null) {
@@ -559,9 +574,10 @@ public class StandardHost
      *
      * @param alias The alias to be added
      */
+    @Override
     public void addAlias(String alias) {
 
-        alias = alias.toLowerCase();
+        alias = alias.toLowerCase(Locale.ENGLISH);
 
         synchronized (aliasesLock) {
             // Skip duplicate aliases
@@ -588,12 +604,10 @@ public class StandardHost
      *
      * @param child Child container to be added
      */
+    @Override
     public void addChild(Container child) {
 
-        if (child instanceof Lifecycle) {
-            ((Lifecycle) child).addLifecycleListener(
-                    new MemoryLeakTrackingListener());
-        }
+        child.addLifecycleListener(new MemoryLeakTrackingListener());
 
         if (!(child instanceof Context))
             throw new IllegalArgumentException
@@ -608,7 +622,7 @@ public class StandardHost
      * is kept of the class loader used every time a context starts.
      */
     private class MemoryLeakTrackingListener implements LifecycleListener {
-
+        @Override
         public void lifecycleEvent(LifecycleEvent event) {
             if (event.getType().equals(Lifecycle.AFTER_START_EVENT)) {
                 if (event.getSource() instanceof Context) {
@@ -650,6 +664,7 @@ public class StandardHost
      * Return the set of alias names for this Host.  If none are defined,
      * a zero length array is returned.
      */
+    @Override
     public String[] findAliases() {
 
         synchronized (aliasesLock) {
@@ -664,6 +679,7 @@ public class StandardHost
      * the corresponding version number, in the format
      * <code>&lt;description&gt;/&lt;version&gt;</code>.
      */
+    @Override
     public String getInfo() {
 
         return (info);
@@ -671,63 +687,15 @@ public class StandardHost
     }
 
 
-    /**
-     * Return the Context that would be used to process the specified
-     * host-relative request URI, if any; otherwise return <code>null</code>.
-     *
-     * @param uri Request URI to be mapped
-     */
-    public Context map(String uri) {
-
-        if (log.isDebugEnabled())
-            log.debug("Mapping request URI '" + uri + "'");
-        if (uri == null)
-            return (null);
-
-        // Match on the longest possible context path prefix
-        if (log.isTraceEnabled())
-            log.trace("  Trying the longest context path prefix");
-        Context context = null;
-        String mapuri = uri;
-        while (true) {
-            context = (Context) findChild(mapuri);
-            if (context != null)
-                break;
-            int slash = mapuri.lastIndexOf('/');
-            if (slash < 0)
-                break;
-            mapuri = mapuri.substring(0, slash);
-        }
-
-        // If no Context matches, select the default Context
-        if (context == null) {
-            if (log.isTraceEnabled())
-                log.trace("  Trying the default context");
-            context = (Context) findChild("");
-        }
-
-        // Complain if no Context has been selected
-        if (context == null) {
-            log.error(sm.getString("standardHost.mappingError", uri));
-            return (null);
-        }
-
-        // Return the mapped Context (if any)
-        if (log.isDebugEnabled())
-            log.debug(" Mapped to context '" + context.getPath() + "'");
-        return (context);
-
-    }
-
-
     /**
      * Remove the specified alias name from the aliases for this Host.
      *
      * @param alias Alias name to be removed
      */
+    @Override
     public void removeAlias(String alias) {
 
-        alias = alias.toLowerCase();
+        alias = alias.toLowerCase(Locale.ENGLISH);
 
         synchronized (aliasesLock) {
 
@@ -762,9 +730,10 @@ public class StandardHost
     /**
      * Return a String representation of this component.
      */
+    @Override
     public String toString() {
 
-        StringBuffer sb = new StringBuffer();
+        StringBuilder sb = new StringBuilder();
         if (getParent() != null) {
             sb.append(getParent().toString());
             sb.append(".");
@@ -775,83 +744,55 @@ public class StandardHost
         return (sb.toString());
 
     }
-
-
+    
     /**
-     * Start this host.
+     * Start this component and implement the requirements
+     * of {@link org.apache.catalina.util.LifecycleBase#startInternal()}.
      *
      * @exception LifecycleException if this component detects a fatal error
-     *  that prevents it from being started
+     *  that prevents this component from being used
      */
-    public synchronized void start() throws LifecycleException {
-        if( started ) {
-            return;
-        }
-        if( ! initialized )
-            init();
-
-        // Look for a realm - that may have been configured earlier. 
-        // If the realm is added after context - it'll set itself.
-        if( realm == null ) {
-            ObjectName realmName=null;
-            try {
-                realmName=new ObjectName( domain + ":type=Realm,host=" + getName());
-                if( mserver.isRegistered(realmName ) ) {
-                    mserver.invoke(realmName, "init", 
-                            new Object[] {},
-                            new String[] {}
-                    );            
-                }
-            } catch( Throwable t ) {
-                log.debug("No realm for this host " + realmName);
-            }
-        }
-            
+    @Override
+    protected synchronized void startInternal() throws LifecycleException {
+        
         // Set error report valve
-        if ((errorReportValveClass != null)
-            && (!errorReportValveClass.equals(""))) {
+        String errorValve = getErrorReportValveClass();
+        if ((errorValve != null) && (!errorValve.equals(""))) {
             try {
                 boolean found = false;
-                if(errorReportValveObjectName != null) {
-                    ObjectName[] names = 
-                        ((StandardPipeline)pipeline).getValveObjectNames();
-                    for (int i=0; !found && i<names.length; i++)
-                        if(errorReportValveObjectName.equals(names[i]))
-                            found = true ;
-                    }
-                    if(!found) {          	
-                        Valve valve = (Valve) Class.forName(errorReportValveClass)
-                        .newInstance();
-                        addValve(valve);
-                        errorReportValveObjectName = ((ValveBase)valve).getObjectName() ;
+                Valve[] valves = getPipeline().getValves();
+                for (Valve valve : valves) {
+                    if (errorValve.equals(valve.getClass().getName())) {
+                        found = true;
+                        break;
                     }
+                }
+                if(!found) {
+                    Valve valve =
+                        (Valve) Class.forName(errorValve).newInstance();
+                    getPipeline().addValve(valve);
+                }
             } catch (Throwable t) {
-                log.error(sm.getString
-                    ("standardHost.invalidErrorReportValveClass", 
-                     errorReportValveClass), t);
+                ExceptionUtils.handleThrowable(t);
+                log.error(sm.getString(
+                        "standardHost.invalidErrorReportValveClass",
+                        errorValve), t);
             }
         }
-        if(log.isDebugEnabled()) {
-            if (xmlValidation)
-                log.debug(sm.getString("standardHost.validationEnabled"));
-            else
-                log.debug(sm.getString("standardHost.validationDisabled"));
-        }
-        super.start();
-
+        super.startInternal();
     }
 
 
     // -------------------- JMX  --------------------
     /**
-      * Return the MBean Names of the Valves assoicated with this Host
+      * Return the MBean Names of the Valves associated with this Host
       *
       * @exception Exception if an MBean cannot be created or registered
       */
      public String [] getValveNames()
          throws Exception
     {
-         Valve [] valves = this.getValves();
+         Valve [] valves = this.getPipeline().getValves();
          String [] mbeanNames = new String[valves.length];
          for (int i = 0; i < valves.length; i++) {
              if( valves[i] == null ) continue;
@@ -869,78 +810,13 @@ public class StandardHost
         }
     }
 
-    private boolean initialized=false;
-    
-    public void init() {
-        if( initialized ) return;
-        initialized=true;
-        
-        // already registered.
-        if( getParent() == null ) {
-            try {
-                // Register with the Engine
-                ObjectName serviceName=new ObjectName(domain + 
-                                        ":type=Engine");
+    @Override
+    protected String getObjectNameKeyProperties() {
 
-                HostConfig deployer = new HostConfig();
-                addLifecycleListener(deployer);                
-                if( mserver.isRegistered( serviceName )) {
-                    if(log.isDebugEnabled())
-                        log.debug("Registering "+ serviceName +" with the Engine");
-                    mserver.invoke( serviceName, "addChild",
-                            new Object[] { this },
-                            new String[] { "org.apache.catalina.Container" } );
-                }
-            } catch( Exception ex ) {
-                log.error("Host registering failed!",ex);
-            }
-        }
-        
-        if( oname==null ) {
-            // not registered in JMX yet - standalone mode
-            try {
-                StandardEngine engine=(StandardEngine)parent;
-                domain=engine.getName();
-                if(log.isDebugEnabled())
-                    log.debug( "Register host " + getName() + " with domain "+ domain );
-                oname=new ObjectName(domain + ":type=Host,host=" +
-                        this.getName());
-                controller = oname;
-                Registry.getRegistry(null, null)
-                    .registerComponent(this, oname, null);
-            } catch( Throwable t ) {
-                log.error("Host registering failed!", t );
-            }
-        }
-    }
+        StringBuilder keyProperties = new StringBuilder("type=Host");
+        keyProperties.append(MBeanUtils.getContainerKeyProperties(this));
 
-    public void destroy() throws Exception {
-        // destroy our child containers, if any
-        Container children[] = findChildren();
-        super.destroy();
-        for (int i = 0; i < children.length; i++) {
-            if(children[i] instanceof StandardContext)
-                ((StandardContext)children[i]).destroy();
-        }
-      
+        return keyProperties.toString();
     }
     
-    public ObjectName preRegister(MBeanServer server, ObjectName oname ) 
-        throws Exception
-    {
-        ObjectName res=super.preRegister(server, oname);
-        String name=oname.getKeyProperty("host");
-        if( name != null )
-            setName( name );
-        return res;        
-    }
-    
-    public ObjectName createObjectName(String domain, ObjectName parent)
-        throws Exception
-    {
-        if( log.isDebugEnabled())
-            log.debug("Create ObjectName " + domain + " " + parent );
-        return new ObjectName( domain + ":type=Host,host=" + getName());
-    }
-
 }
