@@ -32,6 +32,7 @@ import org.apache.tomcat.util.res.StringManager;
  * no-op destruction. 
  * 
  * @author xxd
+ *
  */
 public abstract class FilterBase implements Filter {
     
@@ -40,13 +41,11 @@ public abstract class FilterBase implements Filter {
 
     protected abstract Log getLogger();
     
+    @Override
     public void init(FilterConfig filterConfig) throws ServletException {
-        
-        @SuppressWarnings("unchecked") // Servlet 2.5 doesn't use generics
-        Enumeration paramNames = filterConfig.getInitParameterNames();
-        
+        Enumeration<String> paramNames = filterConfig.getInitParameterNames();
         while (paramNames.hasMoreElements()) {
-            String paramName = (String) paramNames.nextElement();
+            String paramName = paramNames.nextElement();
             if (!IntrospectionUtils.setProperty(this, paramName,
                     filterConfig.getInitParameter(paramName))) {
                 String msg = sm.getString("filterbase.noSuchProperty",
@@ -60,6 +59,7 @@ public abstract class FilterBase implements Filter {
         }    
     }
 
+    @Override
     public void destroy() {
         // NOOP
     }
