@@ -53,7 +53,7 @@ import org.apache.tomcat.util.res.StringManager;
  * track of deltas during a request.
  *
  * @author Filip Hanik
- * @version $Id: DeltaSession.java 1175194 2011-09-24 16:32:19Z rjung $
+ * @version $Id: DeltaSession.java 1226162 2011-12-31 18:32:53Z markt $
  */
 
 public class DeltaSession extends StandardSession implements Externalizable,ClusterSession,ReplicatedMapEntry {
@@ -263,7 +263,7 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
      */
     @Override
     public void setId(String id) {
-        super.setId(id);
+        super.setId(id, true);
         resetDeltaRequest();
     }
 
@@ -276,16 +276,12 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
 
     public void setMaxInactiveInterval(int interval, boolean addDeltaRequest) {
         super.maxInactiveInterval = interval;
-        if (isValid && interval == 0) {
-            expire();
-        } else {
-            if (addDeltaRequest && (deltaRequest != null)) {
-                try {
-                    lock();
-                    deltaRequest.setMaxInactiveInterval(interval);
-                }finally{
-                    unlock();
-                }
+        if (addDeltaRequest && (deltaRequest != null)) {
+            try {
+                lock();
+                deltaRequest.setMaxInactiveInterval(interval);
+            }finally{
+                unlock();
             }
         }
     }
@@ -374,7 +370,7 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
         if (ACTIVITY_CHECK && accessCount.get() > 0) {
             return true;
         }
-        if (maxInactiveInterval >= 0) {
+        if (maxInactiveInterval > 0) {
             long timeNow = System.currentTimeMillis();
             int timeIdle;
             if (LAST_ACCESS_AT_START) {
