@@ -34,16 +34,16 @@ import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 
 import org.apache.catalina.connector.Request;
-import org.apache.catalina.util.RequestUtil;
 import org.apache.coyote.Constants;
 import org.apache.tomcat.util.buf.B2CConverter;
+import org.apache.tomcat.util.http.RequestUtil;
 
 /**
  * An implementation of SSIExternalResolver that is used with servlets.
  * 
  * @author Dan Sandberg
  * @author David Becker
- * @version $Id: SSIServletExternalResolver.java 1140463 2011-06-28 07:22:28Z markt $
+ * @version $Id: SSIServletExternalResolver.java 1239053 2012-02-01 10:52:00Z markt $
  */
 public class SSIServletExternalResolver implements SSIExternalResolver {
     protected final String VARIABLE_NAMES[] = {"AUTH_TYPE", "CONTENT_LENGTH",
