@@ -23,6 +23,7 @@ import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Stack;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 
@@ -54,7 +55,6 @@ import org.apache.catalina.core.StandardWrapper;
 import org.apache.catalina.deploy.LoginConfig;
 import org.apache.catalina.realm.GenericPrincipal;
 import org.apache.catalina.realm.RealmBase;
-import org.apache.catalina.session.StandardManager;
 
 // TODO: lazy init for the temp dir - only when a JSP is compiled or 
 // get temp dir is called we need to create it. This will avoid the 
@@ -83,6 +83,35 @@ import org.apache.catalina.session.StandardManager;
  * use if you have a webapp with a web.xml file, but it is 
  * optional - you can use your own servlets.
  * 
+ * There are a variety of 'add' methods to configure servlets and webapps. These
+ * methods, by default, create a simple in-memory security realm and apply it.
+ * If you need more complex security processing, you can define a subclass of
+ * this class.
+ * 
+ * This class provides a set of convenience methods for configuring webapp
+ * contexts, all overloads of the method <pre>addWebapp</pre>. These methods
+ * create a webapp context, configure it, and then add it to a {@link Host}.
+ * They do not use a global default web.xml; rather, they add a lifecycle
+ * listener that adds the standard DefaultServlet, JSP processing, and welcome
+ * files.
+ * 
+ * In complex cases, you may prefer to use the ordinary Tomcat API to create
+ * webapp contexts; for example, you might need to install a custom Loader
+ * before the call to {@link Host#addChild(Container)}. To replicate the basic
+ * behavior of the <pre>addWebapp</pre> methods, you may want to call two
+ * methods of this class: {@link #noDefaultWebXmlPath()} and
+ * {@link #getDefaultWebXmlListener()}. 
+ * 
+ * {@link #getDefaultRealm()} returns the simple security realm.
+ * 
+ * {@link #getDefaultWebXmlListener()} returns a {@link LifecycleListener} that
+ * adds the standard DefaultServlet, JSP processing, and welcome files. If you
+ * add this listener, you must prevent Tomcat from applying any standard global
+ * web.xml with ...
+ * 
+ * {@link #noDefaultWebXmlPath()} returns a dummy pathname to configure to
+ * prevent {@link ContextConfig} from trying to apply a global web.xml file. 
+ * 
  * This class provides a main() and few simple CLI arguments,
  * see setters for doc. It can be used for simple tests and
  * demo.
@@ -111,14 +140,15 @@ public class Tomcat {
     protected String hostname = "localhost";
     protected String basedir;
     
-    // Default in-memory realm, will be set by default on 
-    // created contexts. Can be replaced with setRealm() on 
-    // the context.
+    // Default in-memory realm, will be set by default on the Engine. Can be
+    // replaced at engine level or over-ridden at Host or Context level
+    @Deprecated // Will be removed in Tomcat 8.0.x.
     protected Realm defaultRealm;
     private Map<String, String> userPass = new HashMap<String, String>();
-    private Map<String, List<String>> userRoles = 
-            new HashMap<String, List<String>>();
-    private Map<String, Principal> userPrincipals = new HashMap<String, Principal>();
+    private Map<String, List<String>> userRoles =
+        new HashMap<String, List<String>>();
+    private Map<String, Principal> userPrincipals =
+        new HashMap<String, Principal>();
     
     public Tomcat() {
         // NOOP
@@ -415,7 +445,10 @@ public class Tomcat {
      * default will be used, using an internal map.
      * 
      * Must be called before adding a context.
+     * 
+     * @deprecated Will be removed in Tomcat 8.0.x.
      */
+    @Deprecated
     public void setDefaultRealm(Realm realm) {
         defaultRealm = realm;
     }
@@ -430,6 +463,10 @@ public class Tomcat {
             engine = new StandardEngine();
             engine.setName( "Tomcat" );
             engine.setDefaultHost(hostname);
+            if (defaultRealm == null) {
+                initSimpleAuth();
+            }
+            engine.setRealm(defaultRealm);
             service.setContainer(engine);
         }
         return engine;
@@ -464,7 +501,7 @@ public class Tomcat {
 
     public Context addContext(Host host, String contextPath, String contextName,
             String dir) {
-        silence(contextPath);
+        silence(host, contextPath);
         Context ctx = new StandardContext();
         ctx.setName(contextName);
         ctx.setPath(contextPath);
@@ -484,16 +521,12 @@ public class Tomcat {
     }
 
     public Context addWebapp(Host host, String url, String name, String path) {
-        silence(url);
+        silence(host, url);
 
         Context ctx = new StandardContext();
         ctx.setName(name);
         ctx.setPath(url);
         ctx.setDocBase(path);
-        if (defaultRealm == null) {
-            initSimpleAuth();
-        }
-        ctx.setRealm(defaultRealm);
 
         ctx.addLifecycleListener(new DefaultWebXmlListener());
         
@@ -501,8 +534,8 @@ public class Tomcat {
         ctx.addLifecycleListener(ctxCfg);
         
         // prevent it from looking ( if it finds one - it'll have dup error )
-        ctxCfg.setDefaultWebXml("org/apache/catalin/startup/NO_DEFAULT_XML");
-        
+        ctxCfg.setDefaultWebXml(noDefaultWebXmlPath());
+
         if (host == null) {
             getHost().addChild(ctx);
         } else {
@@ -511,14 +544,53 @@ public class Tomcat {
 
         return ctx;
     }
+    
+    /**
+     * Return a listener that provides the required configuration items for JSP
+     * processing. From the standard Tomcat global web.xml. Pass this to
+     * {@link Context#addLifecycleListener(LifecycleListener)} and then pass the
+     * result of {@link #noDefaultWebXmlPath()} to 
+     * {@link ContextConfig#setDefaultWebXml(String)}. 
+     * @return a listener object that configures default JSP processing.
+     */
+    public LifecycleListener getDefaultWebXmlListener() {
+        return new DefaultWebXmlListener();
+    }
+    
+    /**
+     * @return a pathname to pass to
+     * {@link ContextConfig#setDefaultWebXml(String)} when using
+     * {@link #getDefaultWebXmlListener()}.
+     */
+    public String noDefaultWebXmlPath() {
+        return Constants.NoDefaultWebXml;
+    }
+    
+    /**
+     * For complex configurations, this accessor allows callers of this class
+     * to obtain the simple realm created by default.
+     * @return the simple in-memory realm created by default.
+     * @deprecated Will be removed in Tomcat 8.0.x
+     */
+    @Deprecated
+    public Realm getDefaultRealm() {
+        if (defaultRealm == null) {
+            initSimpleAuth();
+        }
+        return defaultRealm;
+    }
 
 
     // ---------- Helper methods and classes -------------------
     
     /** 
-     * Initialize an in-memory realm. You can replace it 
-     * for contexts with a real one. 
+     * Create an in-memory realm. You can replace it for contexts with a real
+     * one. The Realm created here will be added to the Engine by default and
+     * may be replaced at the Engine level or over-ridden (as per normal Tomcat
+     * behaviour) at the Host or Context level.
+     * @deprecated Will be removed in Tomcat 8.0.x
      */
+    @Deprecated
     protected void initSimpleAuth() {
         defaultRealm = new RealmBase() {
             @Override
@@ -549,11 +621,12 @@ public class Tomcat {
     }
     
     protected void initBaseDir() {
+        String catalinaHome = System.getProperty(Globals.CATALINA_HOME_PROP);
         if (basedir == null) {
             basedir = System.getProperty(Globals.CATALINA_BASE_PROP);
         }
         if (basedir == null) {
-            basedir = System.getProperty(Globals.CATALINA_HOME_PROP);
+            basedir = catalinaHome;
         }
         if (basedir == null) {
             // Create a temp dir.
@@ -569,7 +642,9 @@ public class Tomcat {
                 }
             }
         }
-        System.setProperty(Globals.CATALINA_HOME_PROP, basedir);
+        if (catalinaHome == null) {
+            System.setProperty(Globals.CATALINA_HOME_PROP, basedir);
+        }
         System.setProperty(Globals.CATALINA_BASE_PROP, basedir);
     }
 
@@ -600,9 +675,13 @@ public class Tomcat {
         }
     }
     
-    private void silence(String ctx) {
+    private void silence(Host host, String ctx) {
         String base = "org.apache.catalina.core.ContainerBase.[default].[";
-        base += getHost().getName();
+        if (host == null) {
+            base += getHost().getName();
+        } else {
+            base += host.getName();
+        }
         base += "].[";
         base += ctx; 
         base += "]";
@@ -680,7 +759,6 @@ public class Tomcat {
         ctx.addServletMapping("*.jspx", "jsp");
 
         // Sessions
-        ctx.setManager( new StandardManager());
         ctx.setSessionTimeout(30);
         
         // MIME mappings
@@ -749,18 +827,35 @@ public class Tomcat {
     public static class ExistingStandardWrapper extends StandardWrapper {
         private Servlet existing;
         boolean init = false;
-        
+
+        @SuppressWarnings("deprecation")
         public ExistingStandardWrapper( Servlet existing ) {
             this.existing = existing;
+            if (existing instanceof javax.servlet.SingleThreadModel) {
+                singleThreadModel = true;
+                instancePool = new Stack<Servlet>();
+            }
         }
         @Override
         public synchronized Servlet loadServlet() throws ServletException {
-            if (!init) {
-                existing.init(facade);
-                init = true;
+            if (singleThreadModel) {
+                Servlet instance;
+                try {
+                    instance = existing.getClass().newInstance();
+                } catch (InstantiationException e) {
+                    throw new ServletException(e);
+                } catch (IllegalAccessException e) {
+                    throw new ServletException(e);
+                }
+                instance.init(facade);
+                return instance;
+            } else {
+                if (!init) {
+                    existing.init(facade);
+                    init = true;
+                }
+                return existing;
             }
-            return existing;
-
         }
         @Override
         public long getAvailable() {
