@@ -5,32 +5,30 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
-
 package org.apache.catalina.valves;
 
 
 import java.io.IOException;
 import java.io.Writer;
 
+import javax.servlet.RequestDispatcher;
 import javax.servlet.ServletException;
 import javax.servlet.http.HttpServletResponse;
 
-import org.apache.catalina.Globals;
 import org.apache.catalina.connector.Request;
 import org.apache.catalina.connector.Response;
 import org.apache.catalina.util.RequestUtil;
 import org.apache.catalina.util.ServerInfo;
-import org.apache.catalina.util.StringManager;
+import org.apache.tomcat.util.ExceptionUtils;
 
 /**
  * <p>Implementation of a Valve that outputs HTML error pages.</p>
@@ -45,12 +43,15 @@ import org.apache.catalina.util.StringManager;
  * @author <a href="mailto:nicolaken@supereva.it">Nicola Ken Barozzi</a> Aisa
  * @author <a href="mailto:stefano@apache.org">Stefano Mazzocchi</a>
  * @author Yoav Shapira
- * @version $Id: ErrorReportValve.java 939353 2010-04-29 15:50:43Z kkolinko $
+ * @version $Id: ErrorReportValve.java 1187022 2011-10-20 19:55:37Z markt $
  */
 
-public class ErrorReportValve
-    extends ValveBase {
+public class ErrorReportValve extends ValveBase {
 
+    //------------------------------------------------------ Constructor
+    public ErrorReportValve() {
+        super(true);
+    }
 
     // ----------------------------------------------------- Instance Variables
 
@@ -62,19 +63,13 @@ public class ErrorReportValve
         "org.apache.catalina.valves.ErrorReportValve/1.0";
 
 
-    /**
-     * The StringManager for this package.
-     */
-    protected static StringManager sm =
-        StringManager.getManager(Constants.Package);
-
-
     // ------------------------------------------------------------- Properties
 
 
     /**
      * Return descriptive information about this Valve implementation.
      */
+    @Override
     public String getInfo() {
 
         return (info);
@@ -95,19 +90,24 @@ public class ErrorReportValve
      * @exception IOException if an input/output error occurs
      * @exception ServletException if a servlet error occurs
      */
+    @Override
     public void invoke(Request request, Response response)
         throws IOException, ServletException {
 
         // Perform the request
         getNext().invoke(request, response);
 
-        Throwable throwable =
-            (Throwable) request.getAttribute(Globals.EXCEPTION_ATTR);
-
         if (response.isCommitted()) {
             return;
         }
 
+        if (request.isAsyncStarted()) {
+            return;
+        }
+
+        Throwable throwable =
+            (Throwable) request.getAttribute(RequestDispatcher.ERROR_EXCEPTION);
+
         if (throwable != null) {
 
             // The response is an error
@@ -117,7 +117,7 @@ public class ErrorReportValve
             try {
                 response.reset();
             } catch (IllegalStateException e) {
-                ;
+                // Ignore
             }
 
             response.sendError
@@ -130,7 +130,7 @@ public class ErrorReportValve
         try {
             report(request, response, throwable);
         } catch (Throwable tt) {
-            ;
+            ExceptionUtils.handleThrowable(tt);
         }
 
     }
@@ -155,24 +155,27 @@ public class ErrorReportValve
 
         // Do nothing on a 1xx, 2xx and 3xx status
         // Do nothing if anything has been written already
-        if ((statusCode < 400) || (response.getContentCount() > 0))
+        if ((statusCode < 400) || (response.getContentWritten() > 0)) {
             return;
+        }
 
         String message = RequestUtil.filter(response.getMessage());
-        if (message == null)
+        if (message == null) {
             message = "";
+        }
 
         // Do nothing if there is no report for the specified status code
         String report = null;
         try {
             report = sm.getString("http." + statusCode, message);
         } catch (Throwable t) {
-            ;
+            ExceptionUtils.handleThrowable(t);
         }
-        if (report == null)
+        if (report == null) {
             return;
+        }
 
-        StringBuffer sb = new StringBuffer();
+        StringBuilder sb = new StringBuilder();
 
         sb.append("<html><head><title>");
         sb.append(ServerInfo.getServerInfo()).append(" - ");
@@ -244,8 +247,10 @@ public class ErrorReportValve
                 response.setContentType("text/html");
                 response.setCharacterEncoding("utf-8");
             } catch (Throwable t) {
-                if (container.getLogger().isDebugEnabled())
+                ExceptionUtils.handleThrowable(t);
+                if (container.getLogger().isDebugEnabled()) {
                     container.getLogger().debug("status.setContentType", t);
+                }
             }
             Writer writer = response.getReporter();
             if (writer != null) {
@@ -254,20 +259,20 @@ public class ErrorReportValve
                 writer.write(sb.toString());
             }
         } catch (IOException e) {
-            ;
+            // Ignore
         } catch (IllegalStateException e) {
-            ;
+            // Ignore
         }
-        
+
     }
 
 
     /**
-     * Print out a partial servlet stack trace (truncating at the last 
+     * Print out a partial servlet stack trace (truncating at the last
      * occurrence of javax.servlet.).
      */
     protected String getPartialServletStackTrace(Throwable t) {
-        StringBuffer trace = new StringBuffer();
+        StringBuilder trace = new StringBuilder();
         trace.append(t.toString()).append('\n');
         StackTraceElement[] elements = t.getStackTrace();
         int pos = elements.length;
@@ -286,5 +291,4 @@ public class ErrorReportValve
         }
         return trace.toString();
     }
-
 }
