@@ -48,7 +48,7 @@ import org.xml.sax.SAXParseException;
  * use a separate class loader for the parser to be used.
  *
  * @author Craig R. McClanahan
- * @version $Id: ParserUtils.java 939520 2010-04-30 00:14:14Z kkolinko $
+ * @version $Id: ParserUtils.java 1229956 2012-01-11 11:04:12Z mturk $
  */
 
 public class ParserUtils {
@@ -73,13 +73,13 @@ public class ParserUtils {
      * Parse the specified XML document, and return a <code>TreeNode</code>
      * that corresponds to the root node of the document tree.
      *
-     * @param uri URI of the XML document being parsed
+     * @param location Location (eg URI) of the XML document being parsed
      * @param is Input source containing the deployment descriptor
      *
      * @exception JasperException if an input/output error occurs
      * @exception JasperException if a parsing error occurs
      */
-    public TreeNode parseXMLDocument(String uri, InputSource is)
+    public TreeNode parseXMLDocument(String location, InputSource is)
         throws JasperException {
 
         Document document = null;
@@ -94,23 +94,23 @@ public class ParserUtils {
             builder.setEntityResolver(entityResolver);
             builder.setErrorHandler(errorHandler);
             document = builder.parse(is);
-	} catch (ParserConfigurationException ex) {
+        } catch (ParserConfigurationException ex) {
             throw new JasperException
-                (Localizer.getMessage("jsp.error.parse.xml", uri), ex);
-	} catch (SAXParseException ex) {
+                (Localizer.getMessage("jsp.error.parse.xml", location), ex);
+        } catch (SAXParseException ex) {
             throw new JasperException
                 (Localizer.getMessage("jsp.error.parse.xml.line",
-				      uri,
-				      Integer.toString(ex.getLineNumber()),
-				      Integer.toString(ex.getColumnNumber())),
-		 ex);
-	} catch (SAXException sx) {
+                                      location,
+                                      Integer.toString(ex.getLineNumber()),
+                                      Integer.toString(ex.getColumnNumber())),
+                 ex);
+        } catch (SAXException sx) {
             throw new JasperException
-                (Localizer.getMessage("jsp.error.parse.xml", uri), sx);
+                (Localizer.getMessage("jsp.error.parse.xml", location), sx);
         } catch (IOException io) {
             throw new JasperException
-                (Localizer.getMessage("jsp.error.parse.xml", uri), io);
-	}
+                (Localizer.getMessage("jsp.error.parse.xml", location), io);
+        }
 
         // Convert the resulting document to a graph of TreeNodes
         return (convert(null, document.getDocumentElement()));
@@ -133,6 +133,17 @@ public class ParserUtils {
         return (parseXMLDocument(uri, new InputSource(is)));
     }
 
+    /**
+     * Set the EntityResolver.
+     * This is needed when the dtds and Jasper itself are in different
+     * classloaders (e.g. OSGi environment).
+     *
+     * @param er EntityResolver to use.
+     */
+    public static void setEntityResolver(EntityResolver er) {
+
+        entityResolver = er;
+    }
 
     // ------------------------------------------------------ Protected Methods
 
@@ -176,7 +187,7 @@ public class ParserUtils {
                             treeNode.setBody(body);
                     }
                 } else {
-                    TreeNode treeChild = convert(treeNode, child);
+                    convert(treeNode, child);
                 }
             }
         }
@@ -191,12 +202,14 @@ public class ParserUtils {
 
 class MyEntityResolver implements EntityResolver {
 
+    @Override
     public InputSource resolveEntity(String publicId, String systemId)
             throws SAXException {
-        for (int i = 0; i < Constants.CACHED_DTD_PUBLIC_IDS.length; i++) {
-            String cachedDtdPublicId = Constants.CACHED_DTD_PUBLIC_IDS[i];
+        for (int i = 0; i < Constants.CACHED_DTD_PUBLIC_IDS.size(); i++) {
+            String cachedDtdPublicId = Constants.CACHED_DTD_PUBLIC_IDS.get(i);
             if (cachedDtdPublicId.equals(publicId)) {
-                String resourcePath = Constants.CACHED_DTD_RESOURCE_PATHS[i];
+                String resourcePath =
+                    Constants.CACHED_DTD_RESOURCE_PATHS.get(i);
                 InputStream input = this.getClass().getResourceAsStream(
                         resourcePath);
                 if (input == null) {
@@ -218,6 +231,7 @@ class MyEntityResolver implements EntityResolver {
 
 class MyErrorHandler implements ErrorHandler {
 
+    @Override
     public void warning(SAXParseException ex) throws SAXException {
         Log log = LogFactory.getLog(MyErrorHandler.class);
         if (log.isDebugEnabled())
@@ -225,10 +239,12 @@ class MyErrorHandler implements ErrorHandler {
         // We ignore warnings
     }
 
+    @Override
     public void error(SAXParseException ex) throws SAXException {
         throw ex;
     }
 
+    @Override
     public void fatalError(SAXParseException ex) throws SAXException {
         throw ex;
     }
