@@ -27,8 +27,11 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Date;
 import java.util.Hashtable;
+import java.util.List;
 
 import javax.naming.NameAlreadyBoundException;
+import javax.naming.NameClassPair;
+import javax.naming.NameNotFoundException;
 import javax.naming.NamingEnumeration;
 import javax.naming.NamingException;
 import javax.naming.OperationNotSupportedException;
@@ -36,22 +39,22 @@ import javax.naming.directory.Attributes;
 import javax.naming.directory.DirContext;
 import javax.naming.directory.ModificationItem;
 import javax.naming.directory.SearchControls;
+import javax.naming.directory.SearchResult;
 
-import org.apache.catalina.util.RequestUtil;
-import org.apache.naming.NamingContextBindingsEnumeration;
 import org.apache.naming.NamingContextEnumeration;
 import org.apache.naming.NamingEntry;
+import org.apache.tomcat.util.http.RequestUtil;
 
 /**
  * Filesystem Directory Context implementation helper class.
  *
  * @author Remy Maucherat
- * @version $Id: FileDirContext.java 939511 2010-04-29 23:48:20Z kkolinko $
+ * @version $Id: FileDirContext.java 1239053 2012-02-01 10:52:00Z markt $
  */
 
 public class FileDirContext extends BaseDirContext {
 
-    private static org.apache.juli.logging.Log log=
+    private static final org.apache.juli.logging.Log log=
         org.apache.juli.logging.LogFactory.getLog( FileDirContext.class );
 
     // -------------------------------------------------------------- Constants
@@ -77,7 +80,7 @@ public class FileDirContext extends BaseDirContext {
     /**
      * Builds a file directory context using the given environment.
      */
-    public FileDirContext(Hashtable env) {
+    public FileDirContext(Hashtable<String,Object> env) {
         super(env);
     }
 
@@ -97,12 +100,6 @@ public class FileDirContext extends BaseDirContext {
     protected String absoluteBase = null;
 
 
-    /**
-     * Case sensitivity.
-     */
-    protected boolean caseSensitive = true;
-
-
     /**
      * Allow linking.
      */
@@ -122,6 +119,7 @@ public class FileDirContext extends BaseDirContext {
      * @exception IllegalArgumentException if this would create a
      *  malformed URL
      */
+    @Override
     public void setDocBase(String docBase) {
 
     // Validate the format of the proposed document root
@@ -147,22 +145,6 @@ public class FileDirContext extends BaseDirContext {
     }
 
 
-    /**
-     * Set case sensitivity.
-     */
-    public void setCaseSensitive(boolean caseSensitive) {
-        this.caseSensitive = caseSensitive;
-    }
-
-
-    /**
-     * Is case sensitive ?
-     */
-    public boolean isCaseSensitive() {
-        return caseSensitive;
-    }
-
-
     /**
      * Set allow linking.
      */
@@ -185,11 +167,24 @@ public class FileDirContext extends BaseDirContext {
     /**
      * Release any resources allocated for this directory context.
      */
+    @Override
     public void release() {
         super.release();
     }
 
 
+    /**
+     * Return the real path for a given virtual path, if possible; otherwise
+     * return <code>null</code>.
+     *
+     * @param path The path to the desired resource
+     */
+    @Override
+    protected String doGetRealPath(String path) {
+        File file = new File(getDocBase(), path);
+        return file.getAbsolutePath();
+    }
+
     // -------------------------------------------------------- Context Methods
 
 
@@ -198,22 +193,19 @@ public class FileDirContext extends BaseDirContext {
      *
      * @param name the name of the object to look up
      * @return the object bound to name
-     * @exception NamingException if a naming exception is encountered
      */
-    public Object lookup(String name)
-        throws NamingException {
+    @Override
+    protected Object doLookup(String name) {
         Object result = null;
         File file = file(name);
 
         if (file == null)
-            throw new NamingException
-                (sm.getString("resources.notFound", name));
+            return null;
 
         if (file.isDirectory()) {
             FileDirContext tempContext = new FileDirContext(env);
             tempContext.setDocBase(file.getPath());
             tempContext.setAllowLinking(getAllowLinking());
-            tempContext.setCaseSensitive(isCaseSensitive());
             result = tempContext;
         } else {
             result = new FileResource(file);
@@ -238,14 +230,15 @@ public class FileDirContext extends BaseDirContext {
      * exist
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public void unbind(String name)
         throws NamingException {
 
         File file = file(name);
 
         if (file == null)
-            throw new NamingException
-                (sm.getString("resources.notFound", name));
+            throw new NameNotFoundException(
+                    sm.getString("resources.notFound", name));
 
         if (!file.delete())
             throw new NamingException
@@ -265,18 +258,22 @@ public class FileDirContext extends BaseDirContext {
      * @exception NameAlreadyBoundException if newName is already bound
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public void rename(String oldName, String newName)
         throws NamingException {
 
         File file = file(oldName);
 
         if (file == null)
-            throw new NamingException
+            throw new NameNotFoundException
                 (sm.getString("resources.notFound", oldName));
 
         File newFile = new File(base, newName);
 
-        file.renameTo(newFile);
+        if (!file.renameTo(newFile)) {
+            throw new NamingException(sm.getString("resources.renameFail",
+                    oldName, newName));
+        }
 
     }
 
@@ -294,13 +291,14 @@ public class FileDirContext extends BaseDirContext {
      * this context. Each element of the enumeration is of type NameClassPair.
      * @exception NamingException if a naming exception is encountered
      */
-    public NamingEnumeration list(String name)
+    @Override
+    public NamingEnumeration<NameClassPair> list(String name)
         throws NamingException {
 
         File file = file(name);
 
         if (file == null)
-            throw new NamingException
+            throw new NameNotFoundException
                 (sm.getString("resources.notFound", name));
 
         return new NamingContextEnumeration(list(file).iterator());
@@ -321,17 +319,16 @@ public class FileDirContext extends BaseDirContext {
      * Each element of the enumeration is of type Binding.
      * @exception NamingException if a naming exception is encountered
      */
-    public NamingEnumeration listBindings(String name)
+    @Override
+    protected List<NamingEntry> doListBindings(String name)
         throws NamingException {
 
         File file = file(name);
 
         if (file == null)
-            throw new NamingException
-                (sm.getString("resources.notFound", name));
-
-        return new NamingContextBindingsEnumeration(list(file).iterator(),
-                this);
+            return null;
+        
+        return list(file);
 
     }
 
@@ -358,9 +355,10 @@ public class FileDirContext extends BaseDirContext {
      * @param name the name of the context to be destroyed; may not be empty
      * @exception NameNotFoundException if an intermediate context does not
      * exist
-     * @exception NotContextException if the name is bound but does not name
-     * a context, or does not name a context of the appropriate type
+     * @exception javax.naming.NotContextException if the name is bound but does
+     * not name a context, or does not name a context of the appropriate type
      */
+    @Override
     public void destroySubcontext(String name)
         throws NamingException {
         unbind(name);
@@ -377,6 +375,7 @@ public class FileDirContext extends BaseDirContext {
      * (if any).
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public Object lookupLink(String name)
         throws NamingException {
         // Note : Links are not supported
@@ -401,6 +400,7 @@ public class FileDirContext extends BaseDirContext {
      * not have the notion of a full name
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public String getNameInNamespace()
         throws NamingException {
         return docBase;
@@ -422,15 +422,15 @@ public class FileDirContext extends BaseDirContext {
      * indicates that none should be retrieved
      * @exception NamingException if a naming exception is encountered
      */
-    public Attributes getAttributes(String name, String[] attrIds)
+    @Override
+    protected Attributes doGetAttributes(String name, String[] attrIds)
         throws NamingException {
 
         // Building attribute list
         File file = file(name);
 
         if (file == null)
-            throw new NamingException
-                (sm.getString("resources.notFound", name));
+            return null;
 
         return new FileResourceAttributes(file);
 
@@ -447,13 +447,14 @@ public class FileDirContext extends BaseDirContext {
      * REPLACE_ATTRIBUTE, REMOVE_ATTRIBUTE
      * @param attrs the attributes to be used for the modification; may not
      * be null
-     * @exception AttributeModificationException if the modification cannot be
-     * completed successfully
+     * @exception javax.naming.directory.AttributeModificationException if the
+     * modification cannot be completed successfully
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public void modifyAttributes(String name, int mod_op, Attributes attrs)
         throws NamingException {
-
+        throw new OperationNotSupportedException();
     }
 
 
@@ -467,13 +468,14 @@ public class FileDirContext extends BaseDirContext {
      * @param name the name of the object whose attributes will be updated
      * @param mods an ordered sequence of modifications to be performed; may
      * not be null
-     * @exception AttributeModificationException if the modification cannot be
-     * completed successfully
+     * @exception javax.naming.directory.AttributeModificationException if the
+     * modification cannot be completed successfully
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public void modifyAttributes(String name, ModificationItem[] mods)
         throws NamingException {
-
+        throw new OperationNotSupportedException();
     }
 
 
@@ -488,10 +490,11 @@ public class FileDirContext extends BaseDirContext {
      * @param obj the object to bind; possibly null
      * @param attrs the attributes to associate with the binding
      * @exception NameAlreadyBoundException if name is already bound
-     * @exception InvalidAttributesException if some "mandatory" attributes
-     * of the binding are not supplied
+     * @exception javax.naming.directory.InvalidAttributesException if some
+     * "mandatory" attributes of the binding are not supplied
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public void bind(String name, Object obj, Attributes attrs)
         throws NamingException {
 
@@ -521,10 +524,11 @@ public class FileDirContext extends BaseDirContext {
      * @param name the name to bind; may not be empty
      * @param obj the object to bind; possibly null
      * @param attrs the attributes to associate with the binding
-     * @exception InvalidAttributesException if some "mandatory" attributes
-     * of the binding are not supplied
+     * @exception javax.naming.directory.InvalidAttributesException if some
+     * "mandatory" attributes of the binding are not supplied
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public void rebind(String name, Object obj, Attributes attrs)
         throws NamingException {
 
@@ -538,6 +542,7 @@ public class FileDirContext extends BaseDirContext {
             try {
                 is = ((Resource) obj).streamContent();
             } catch (IOException e) {
+                // Ignore
             }
         } else if (obj instanceof InputStream) {
             is = (InputStream) obj;
@@ -597,10 +602,11 @@ public class FileDirContext extends BaseDirContext {
      * @param attrs the attributes to associate with the newly created context
      * @return the newly created context
      * @exception NameAlreadyBoundException if the name is already bound
-     * @exception InvalidAttributesException if attrs does not contain all
-     * the mandatory attributes required for creation
+     * @exception javax.naming.directory.InvalidAttributesException if attrs
+     * does not contain all the mandatory attributes required for creation
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public DirContext createSubcontext(String name, Attributes attrs)
         throws NamingException {
 
@@ -629,6 +635,7 @@ public class FileDirContext extends BaseDirContext {
      * @exception OperationNotSupportedException if schema not supported
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public DirContext getSchema(String name)
         throws NamingException {
         throw new OperationNotSupportedException();
@@ -646,6 +653,7 @@ public class FileDirContext extends BaseDirContext {
      * @exception OperationNotSupportedException if schema not supported
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public DirContext getSchemaClassDefinition(String name)
         throws NamingException {
         throw new OperationNotSupportedException();
@@ -669,8 +677,9 @@ public class FileDirContext extends BaseDirContext {
      * context named by name.
      * @exception NamingException if a naming exception is encountered
      */
-    public NamingEnumeration search(String name, Attributes matchingAttributes,
-                                    String[] attributesToReturn)
+    @Override
+    public NamingEnumeration<SearchResult> search(String name,
+            Attributes matchingAttributes, String[] attributesToReturn)
         throws NamingException {
         return null;
     }
@@ -691,8 +700,9 @@ public class FileDirContext extends BaseDirContext {
      * context named by name.
      * @exception NamingException if a naming exception is encountered
      */
-    public NamingEnumeration search(String name, Attributes matchingAttributes)
-        throws NamingException {
+    @Override
+    public NamingEnumeration<SearchResult> search(String name,
+            Attributes matchingAttributes) throws NamingException {
         return null;
     }
 
@@ -710,13 +720,15 @@ public class FileDirContext extends BaseDirContext {
      * (new SearchControls())).
      * @return an enumeration of SearchResults of the objects that satisfy
      * the filter; never null
-     * @exception InvalidSearchFilterException if the search filter specified
-     * is not supported or understood by the underlying directory
-     * @exception InvalidSearchControlsException if the search controls
-     * contain invalid settings
+     * @exception javax.naming.directory.InvalidSearchFilterException if the
+     * search filter specified is not supported or understood by the underlying
+     * directory
+     * @exception javax.naming.directory.InvalidSearchControlsException if the
+     * search controls contain invalid settings
      * @exception NamingException if a naming exception is encountered
      */
-    public NamingEnumeration search(String name, String filter,
+    @Override
+    public NamingEnumeration<SearchResult> search(String name, String filter,
                                     SearchControls cons)
         throws NamingException {
         return null;
@@ -737,18 +749,19 @@ public class FileDirContext extends BaseDirContext {
      * occurrence of "{i}". If null, equivalent to an empty array.
      * @param cons the search controls that control the search. If null, the
      * default search controls are used (equivalent to (new SearchControls())).
-     * @return an enumeration of SearchResults of the objects that satisy the
+     * @return an enumeration of SearchResults of the objects that satisfy the
      * filter; never null
      * @exception ArrayIndexOutOfBoundsException if filterExpr contains {i}
      * expressions where i is outside the bounds of the array filterArgs
-     * @exception InvalidSearchControlsException if cons contains invalid
-     * settings
-     * @exception InvalidSearchFilterException if filterExpr with filterArgs
-     * represents an invalid search filter
+     * @exception javax.naming.directory.InvalidSearchControlsException if cons
+     * contains invalid settings
+     * @exception javax.naming.directory.InvalidSearchFilterException if
+     * filterExpr with filterArgs represents an invalid search filter
      * @exception NamingException if a naming exception is encountered
      */
-    public NamingEnumeration search(String name, String filterExpr,
-                                    Object[] filterArgs, SearchControls cons)
+    @Override
+    public NamingEnumeration<SearchResult> search(String name,
+            String filterExpr, Object[] filterArgs, SearchControls cons)
         throws NamingException {
         return null;
     }
@@ -784,14 +797,15 @@ public class FileDirContext extends BaseDirContext {
         File file = new File(base, name);
         if (file.exists() && file.canRead()) {
 
-        	if (allowLinking)
-        		return file;
-        	
+            if (allowLinking)
+                return file;
+            
             // Check that this file belongs to our root path
             String canPath = null;
             try {
                 canPath = file.getCanonicalPath();
             } catch (IOException e) {
+                // Ignore
             }
             if (canPath == null)
                 return null;
@@ -801,27 +815,24 @@ public class FileDirContext extends BaseDirContext {
                 return null;
             }
 
-            // Case sensitivity check
-            if (caseSensitive) {
-                String fileAbsPath = file.getAbsolutePath();
-                if (fileAbsPath.endsWith("."))
-                    fileAbsPath = fileAbsPath + "/";
-                String absPath = normalize(fileAbsPath);
-                if (canPath != null)
-                    canPath = normalize(canPath);
-                if ((absoluteBase.length() < absPath.length())
-                    && (absoluteBase.length() < canPath.length())) {
-                    absPath = absPath.substring(absoluteBase.length() + 1);
-                    if ((canPath == null) || (absPath == null))
-                        return null;
-                    if (absPath.equals(""))
-                        absPath = "/";
-                    canPath = canPath.substring(absoluteBase.length() + 1);
-                    if (canPath.equals(""))
-                        canPath = "/";
-                    if (!canPath.equals(absPath))
-                        return null;
-                }
+            // Case sensitivity check - this is now always done
+            String fileAbsPath = file.getAbsolutePath();
+            if (fileAbsPath.endsWith("."))
+                fileAbsPath = fileAbsPath + "/";
+            String absPath = normalize(fileAbsPath);
+            canPath = normalize(canPath);
+            if ((absoluteBase.length() < absPath.length())
+                && (absoluteBase.length() < canPath.length())) {
+                absPath = absPath.substring(absoluteBase.length() + 1);
+                if (absPath == null)
+                    return null;
+                if (absPath.equals(""))
+                    absPath = "/";
+                canPath = canPath.substring(absoluteBase.length() + 1);
+                if (canPath.equals(""))
+                    canPath = "/";
+                if (!canPath.equals(absPath))
+                    return null;
             }
 
         } else {
@@ -836,11 +847,11 @@ public class FileDirContext extends BaseDirContext {
      * List the resources which are members of a collection.
      *
      * @param file Collection
-     * @return Vector containg NamingEntry objects
+     * @return Vector containing NamingEntry objects
      */
-    protected ArrayList list(File file) {
+    protected List<NamingEntry> list(File file) {
 
-        ArrayList entries = new ArrayList();
+        List<NamingEntry> entries = new ArrayList<NamingEntry>();
         if (!file.isDirectory())
             return entries;
         String[] names = file.list();
@@ -853,8 +864,6 @@ public class FileDirContext extends BaseDirContext {
         }
 
         Arrays.sort(names);             // Sort alphabetically
-        if (names == null)
-            return entries;
         NamingEntry entry = null;
 
         for (int i = 0; i < names.length; i++) {
@@ -865,7 +874,6 @@ public class FileDirContext extends BaseDirContext {
                 FileDirContext tempContext = new FileDirContext(env);
                 tempContext.setDocBase(file.getPath());
                 tempContext.setAllowLinking(getAllowLinking());
-                tempContext.setCaseSensitive(isCaseSensitive());
                 object = tempContext;
             } else {
                 object = new FileResource(currentFile);
@@ -887,7 +895,7 @@ public class FileDirContext extends BaseDirContext {
      * This specialized resource implementation avoids opening the InputStream
      * to the file right away (which would put a lock on the file).
      */
-    protected class FileResource extends Resource {
+    protected static class FileResource extends Resource {
 
 
         // -------------------------------------------------------- Constructor
@@ -907,12 +915,6 @@ public class FileDirContext extends BaseDirContext {
         protected File file;
 
 
-        /**
-         * File length.
-         */
-        protected long length = -1L;
-
-
         // --------------------------------------------------- Resource Methods
 
 
@@ -921,6 +923,7 @@ public class FileDirContext extends BaseDirContext {
          *
          * @return InputStream
          */
+        @Override
         public InputStream streamContent()
             throws IOException {
             if (binaryContent == null) {
@@ -943,12 +946,12 @@ public class FileDirContext extends BaseDirContext {
      * reading (to speed up simple checks, like checking the last modified
      * date).
      */
-    protected class FileResourceAttributes extends ResourceAttributes {
+    protected static class FileResourceAttributes extends ResourceAttributes {
 
+        private static final long serialVersionUID = 1L;
 
         // -------------------------------------------------------- Constructor
 
-
         public FileResourceAttributes(File file) {
             this.file = file;
             getCreation();
@@ -973,6 +976,7 @@ public class FileDirContext extends BaseDirContext {
         /**
          * Is collection.
          */
+        @Override
         public boolean isCollection() {
             if (!accessed) {
                 collection = file.isDirectory();
@@ -987,6 +991,7 @@ public class FileDirContext extends BaseDirContext {
          *
          * @return content length value
          */
+        @Override
         public long getContentLength() {
             if (contentLength != -1L)
                 return contentLength;
@@ -1000,6 +1005,7 @@ public class FileDirContext extends BaseDirContext {
          *
          * @return creation time value
          */
+        @Override
         public long getCreation() {
             if (creation != -1L)
                 return creation;
@@ -1013,6 +1019,7 @@ public class FileDirContext extends BaseDirContext {
          *
          * @return Creation date value
          */
+        @Override
         public Date getCreationDate() {
             if (creation == -1L) {
                 creation = getCreation();
@@ -1026,6 +1033,7 @@ public class FileDirContext extends BaseDirContext {
          *
          * @return lastModified time value
          */
+        @Override
         public long getLastModified() {
             if (lastModified != -1L)
                 return lastModified;
@@ -1039,6 +1047,7 @@ public class FileDirContext extends BaseDirContext {
          *
          * @return LastModified date value
          */
+        @Override
         public Date getLastModifiedDate() {
             if (lastModified == -1L) {
                 lastModified = getLastModified();
@@ -1052,6 +1061,7 @@ public class FileDirContext extends BaseDirContext {
          *
          * @return Name value
          */
+        @Override
         public String getName() {
             if (name == null)
                 name = file.getName();
@@ -1064,6 +1074,7 @@ public class FileDirContext extends BaseDirContext {
          *
          * @return String resource type
          */
+        @Override
         public String getResourceType() {
             if (!accessed) {
                 collection = file.isDirectory();
@@ -1078,6 +1089,7 @@ public class FileDirContext extends BaseDirContext {
          * 
          * @return String the file's canonical path
          */
+        @Override
         public String getCanonicalPath() {
             if (canonicalPath == null) {
                 try {
