@@ -20,6 +20,7 @@ package org.apache.tomcat.util.net;
 
 import java.io.Serializable;
 import java.net.MalformedURLException;
+import java.util.Locale;
 
 
 /**
@@ -41,15 +42,16 @@ import java.net.MalformedURLException;
  * package someplace.</p>
  *
  * @author Craig R. McClanahan
- * @version $Revision: 467222 $ $Date: 2006-10-24 05:17:11 +0200 (Di, 24. Okt 2006) $
+ * @version $Id: URL.java 1200170 2011-11-10 05:54:22Z kkolinko $
  */
 
 public final class URL implements Serializable {
 
+    private static final long serialVersionUID = 1L;
+
 
     // ----------------------------------------------------------- Constructors
 
-
     /**
      * Create a URL object from the specified String representation.
      *
@@ -107,10 +109,10 @@ public final class URL implements Serializable {
             }
 
             // Parse out the new protocol
-            for (i = start; !aRef && (i < limit) ; i++) { 
+            for (i = start; !aRef && (i < limit) ; i++) {
                 c = spec.charAt(i);
                 if (c == ':') {
-                    String s = spec.substring(start, i).toLowerCase();
+                    String s = spec.substring(start, i).toLowerCase(Locale.ENGLISH);
                     // Assume all protocols are valid
                     newProtocol = s;
                     start = i + 1;
@@ -173,9 +175,6 @@ public final class URL implements Serializable {
     }
 
 
-
-
-
     /**
      * Create a URL object from the specified components.  The default port
      * number for the specified protocol will be used.
@@ -302,10 +301,9 @@ public final class URL implements Serializable {
      *
      * @param obj The URL to compare against
      */
+    @Override
     public boolean equals(Object obj) {
 
-        if (obj == null)
-            return (false);
         if (!(obj instanceof URL))
             return (false);
         URL other = (URL) obj;
@@ -316,6 +314,20 @@ public final class URL implements Serializable {
     }
 
 
+    @Override
+    public int hashCode() {
+        final int prime = 31;
+        int result = 1;
+        result = prime * result + ((file == null) ? 0 : file.hashCode());
+        result = prime * result + ((host == null) ? 0 : host.hashCode());
+        result = prime * result + port;
+        result = prime * result +
+                ((protocol == null) ? 0 : protocol.hashCode());
+        result = prime * result + ((ref == null) ? 0 : ref.hashCode());
+        return result;
+    }
+
+
     /**
      * Return the authority part of the URL.
      */
@@ -534,7 +546,7 @@ public final class URL implements Serializable {
      */
     public String toExternalForm() {
 
-        StringBuffer sb = new StringBuffer();
+        StringBuilder sb = new StringBuilder();
         if (protocol != null) {
             sb.append(protocol);
             sb.append(":");
@@ -561,9 +573,10 @@ public final class URL implements Serializable {
     /**
      * Return a string representation of this object.
      */
+    @Override
     public String toString() {
 
-        StringBuffer sb = new StringBuffer("URL[");
+        StringBuilder sb = new StringBuilder("URL[");
         sb.append("authority=");
         sb.append(authority);
         sb.append(", file=");
@@ -671,7 +684,7 @@ public final class URL implements Serializable {
                         at = ipv6-1;
                     }
                 }
-                                                        
+
                 int colon = authority.indexOf(':', at+1);
                 if (colon >= 0) {
                     try {
