@@ -793,6 +793,32 @@ public interface PoolConfiguration {
      * false if it is to be ignored.
      */
     public void setAlternateUsernameAllowed(boolean alternateUsernameAllowed);
+    /**
+     * Set to true if you want the connection pool to commit any pending transaction when a connection is returned.
+     * The default value is false, as this could result in committing data.
+     * This parameter is only looked at if the {@link java.sql.Connection#getAutoCommit()} returns false
+     * @param commitOnReturn set to true if the pool should call {@link java.sql.Connection#commit()} when a connection is returned to the pool.
+     * Default is false
+     */
+    public void setCommitOnReturn(boolean commitOnReturn);
 
+    /**
+     * @see PoolConfiguration#setCommitOnReturn(boolean)
+     */
+    public boolean getCommitOnReturn();
 
-}
\ No newline at end of file
+    /**
+     * Set to true if you want the connection pool to rollback any pending transaction when a connection is returned.
+     * The default value is false, as this could result in committing data.
+     * This parameter is only looked at if the {@link java.sql.Connection#getAutoCommit()} returns false
+     * @param rollbackOnReturn set to true if the pool should call {@link java.sql.Connection#rollback()} when a connection is returned to the pool.
+     * Default is false
+     */
+    public void setRollbackOnReturn(boolean rollbackOnReturn);
+
+    /**
+     * @see PoolConfiguration#setRollbackOnReturn(boolean)
+     */
+    public boolean getRollbackOnReturn();
+
+}
